#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <configfile.h>
#include "prefs_dialog.h"

// Some file names, etc.
#define CONFIG_FILE "/.xmms/config"
#define SECTION_NAME "XMMS Terrain"


// Default values
static float land_diffuse[] = { 0.65f, 0.46f, 0.14f };
static int default_subdivision = 1;

// Widgets we need to read from
static GtkWidget *terrain_color = NULL;
static GtkWidget *geom_spinbutton = NULL;
static GtkWidget *prefs_dialog = NULL;

// Global configuration
XTPConfig xtp_global_config;


//
// Reads the configuration from file
//
void xtp_read_config()
{
  ConfigFile *cfg;
  gchar *filename;

  // populate from defaults
  memcpy(xtp_global_config.terrain_clr, land_diffuse, 3 * sizeof(float));
  xtp_global_config.subdivision = default_subdivision;
  
  filename = g_strconcat(g_get_home_dir(), CONFIG_FILE, NULL);
  cfg = xmms_cfg_open_file(filename);
  
  if (cfg) {
    xmms_cfg_read_float(cfg, SECTION_NAME, "terrain_clr_0",
			&xtp_global_config.terrain_clr[0]);		
    xmms_cfg_read_float(cfg, SECTION_NAME, "terrain_clr_1",
			&xtp_global_config.terrain_clr[1]);		
    xmms_cfg_read_float(cfg, SECTION_NAME, "terrain_clr_2",
			&xtp_global_config.terrain_clr[2]);		
    xmms_cfg_read_int(cfg, SECTION_NAME, "subdivision",
		      &xtp_global_config.subdivision);
    xmms_cfg_free(cfg);
  }
  g_free(filename);
}


//
// Creates the preferences dialog window
//
GtkWidget* xtp_create_prefs_dialog (void)
{
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *table;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *hseparator1;
  GtkWidget *hseparator2;
  GtkObject *geom_spinbutton_adj; 
  GtkWidget *dialog_action_area1;
  GtkWidget *button_ok;
  GtkWidget *button_apply;
  GtkWidget *button_cancel;

  if (prefs_dialog) return NULL;

  // load the configuration
  xtp_read_config();

  // set up the dialog box
  prefs_dialog = gtk_dialog_new ();
  if (!prefs_dialog) return NULL;
  gtk_window_set_title (GTK_WINDOW (prefs_dialog), "XMMS Terrain Preferences");
  gtk_window_set_position (GTK_WINDOW (prefs_dialog), GTK_WIN_POS_CENTER);

  dialog_vbox1 = GTK_DIALOG (prefs_dialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  // Add the "Terrain Color" label
  label1 = gtk_label_new ("Terrain Color");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  // add a secparator
  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator2, TRUE, TRUE, 0);

  // add the terrain color selector
  gdouble clr[4];
  clr[0] = xtp_global_config.terrain_clr[0];
  clr[1] = xtp_global_config.terrain_clr[1];
  clr[2] = xtp_global_config.terrain_clr[2];
  terrain_color = gtk_color_selection_new ();
  gtk_widget_show (terrain_color);
  gtk_box_pack_start (GTK_BOX (vbox1), terrain_color, FALSE, TRUE, 0);
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(terrain_color), clr);

  // add a separator
  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 0);

  // add the "other preferences" table
  table = gtk_table_new (1, 2, FALSE);
  gtk_widget_show (table);
  gtk_box_pack_start (GTK_BOX (vbox1), table, TRUE, TRUE, 0);

  // add the "Geometry Refinement" label
  label2 = gtk_label_new ("Geometry Refinement");
  //gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table), label2, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label2), 60, 0);

  // add the "Geometry Refinement" spinbutton
  geom_spinbutton_adj = gtk_adjustment_new (0, 0, 4, 1, 1, 1);
  geom_spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (geom_spinbutton_adj), 1, 0);
  //gtk_widget_show (geom_spinbutton);
  gtk_table_attach (GTK_TABLE (table), geom_spinbutton, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (geom_spinbutton), TRUE);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(geom_spinbutton), 
			    (gfloat)xtp_global_config.subdivision);
  

  // add the action area
  dialog_action_area1 = GTK_DIALOG (prefs_dialog)->action_area;
  //  gtk_widget_show (dialog_action_area1);
  //  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
 
  // add the "ok" button
  button_ok = gtk_button_new_with_label("OK");
  gtk_widget_show (button_ok);
  gtk_container_add(GTK_CONTAINER(dialog_action_area1), button_ok);
  GTK_WIDGET_SET_FLAGS (button_ok, GTK_CAN_DEFAULT);

  // add the "cancel" button
  button_cancel = gtk_button_new_with_label("Cancel");
  gtk_widget_show (button_cancel);
  gtk_container_add(GTK_CONTAINER(dialog_action_area1), button_cancel);
  GTK_WIDGET_SET_FLAGS (button_cancel, GTK_CAN_DEFAULT);

  //add the "apply" button
  button_apply = gtk_button_new_with_label("Apply");
  gtk_widget_show (button_apply);
  gtk_container_add(GTK_CONTAINER(dialog_action_area1), button_apply);
  GTK_WIDGET_SET_FLAGS (button_apply, GTK_CAN_DEFAULT);


  // set up the callbacks
  gtk_signal_connect (GTK_OBJECT(button_cancel), "clicked", xtp_on_cancel, NULL);
  gtk_signal_connect (GTK_OBJECT(button_apply), "clicked", xtp_on_apply, NULL);
  gtk_signal_connect (GTK_OBJECT(button_ok), "clicked", xtp_on_ok, NULL);

  return prefs_dialog;
}


//
// "OK" button callback
//
void xtp_on_ok(GtkButton *button, gpointer user_data)
{
  // First apply the changes - read them from widgets to the xtp_global_config
  xtp_on_apply(button, user_data);

  // Next, write the configuration
  ConfigFile *cfg;	
  gchar *filename;
	
  filename = g_strconcat(g_get_home_dir(), CONFIG_FILE, NULL);
  cfg = xmms_cfg_open_file(filename);
  if (!cfg) cfg = xmms_cfg_new();

  xmms_cfg_write_float(cfg, SECTION_NAME, "terrain_clr_0",
		       xtp_global_config.terrain_clr[0]);		
  xmms_cfg_write_float(cfg, SECTION_NAME, "terrain_clr_1",
		       xtp_global_config.terrain_clr[1]);		
  xmms_cfg_write_float(cfg, SECTION_NAME, "terrain_clr_2",
		       xtp_global_config.terrain_clr[2]);		
  xmms_cfg_write_int(cfg, SECTION_NAME, "subdivision",
		     xtp_global_config.subdivision);
  xmms_cfg_write_file(cfg, filename);
  xmms_cfg_free(cfg);
  g_free(filename);
  
  gtk_widget_destroy(prefs_dialog);
  prefs_dialog = NULL;
}


//
// Reads values from GTK Widgets into xtp_global_config
//
void xtp_on_apply(GtkButton *button, gpointer user_data)
{
  gdouble clr[4];

  // Grab ground color
  gtk_color_selection_get_color(GTK_COLOR_SELECTION(terrain_color), clr);
  xtp_global_config.terrain_clr[0] = (float)clr[0];
  xtp_global_config.terrain_clr[1] = (float)clr[1];
  xtp_global_config.terrain_clr[2] = (float)clr[2];

  // Grab the geometry refine value
  xtp_global_config.subdivision = (int)
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(geom_spinbutton));
}


void xtp_on_cancel(GtkButton *button, gpointer user_data)
{
  // Reload the config from file
  xtp_read_config();
  gtk_widget_destroy(prefs_dialog);
  prefs_dialog = NULL;
}
